@@ -131,7 +131,7 @@ until [ -z "$1" ]; do
       "-i") DO_INSTALL="yes";;
       "-j") shift; export MAKEFLAGS="$MAKEFLAGS -j$1";;
       "-l") shift; SPEC_LDIR="$1";;
-      "-r") shift; SPEC_RPATH="$1";;
+      "-r") shift; SPEC_RPATH="-DFALCON_RPATH:STRING=\"$1\"";;
       "-int") USE_INTERNAL_LIBS="no";;
       "-b") SKIP_BISON="";;
       "-d") DEBUG="yes";;
@@ -166,7 +166,7 @@ fi
 FALCON_BUILD_TREE=$FALCON_ACTIVE_TREE/build
 
 
-# if already configured..
+# if already configured.
 if [ -z "$TARGET_DEST" ]; then
   if [ -e "$FALCON_BUILD_TREE/Makefile" ]; then 
      func_complete_inst
@@ -206,6 +206,7 @@ mkdir -p $FALCON_BUILD_TREE
 cd $FALCON_BUILD_TREE || func_error
 
 cmake -Wno-dev \
+      -DFALCON_WITH_FEATHERS="../modules/feathers" \
       -DCMAKE_BUILD_TYPE:STRING="$FALCON_BUILD_TYPE" \
       -DCMAKE_INSTALL_PREFIX:STRING="$TARGET_DEST" \
       -DFALCON_INSTALL_TREE:STRING="$TARGET_DEST" \
@@ -215,12 +216,12 @@ cmake -Wno-dev \
       -DFALCON_WITH_INTERNAL_EDITLINE:BOOL=$USE_EDITLINE \
       -DFALCON_LIB_DIR:STRING="$SPEC_LDIR" \
       -DFALCON_INSTALL_TREE_LIB:STRING="$SPEC_LDIR" \
-      -DWITH_INTERNAL_PCRE:BOOL=$USE_INTERNAL_LIBS \
-      -DWITH_INTERNAL_ZLIB:BOOL=$USE_INTERNAL_LIBS \
+      -DFALCON_WITH_INTERNAL_PCRE:BOOL=$USE_INTERNAL_LIBS \
+      -DFALCON_WITH_INTERNAL_ZLIB:BOOL=$USE_INTERNAL_LIBS \
       -DFALCON_WITH_MANPAGES:BOOL=ON \
-      -DFALCON_RPATH:STRING=$SPEC_RPATH \
+      $SPEC_RPATH \
       $EXTRA_CMAKE \
-    $FALCON_SRC_TREE || func_errors
+    $FALCON_SRC_TREE/core || func_errors
 
 if [ "$DO_INSTALL" = "yes" ]; then
    func_complete_inst
