@@ -55,6 +55,8 @@ FALCON_FUNC  mth_metaclass ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_getProperty( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_setProperty( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_properties( ::Falcon::VMachine *vm );
+FALCON_FUNC  Dictionary_dop( ::Falcon::VMachine *vm );
+
 //FALCON_FUNC  mth_hasProperty( ::Falcon::VMachine *vm );
 
 FALCON_FUNC  Function_name ( ::Falcon::VMachine *vm );
@@ -412,6 +414,7 @@ FALCON_FUNC  fileChown ( ::Falcon::VMachine *vm );
 FALCON_FUNC  fileChgroup ( ::Falcon::VMachine *vm );
 FALCON_FUNC  fileCopy ( ::Falcon::VMachine *vm );
 
+FALCON_FUNC  flc_Random_init( ::Falcon::VMachine *vm );
 FALCON_FUNC  flc_random ( ::Falcon::VMachine *vm );
 FALCON_FUNC  flc_randomChoice ( ::Falcon::VMachine *vm );
 FALCON_FUNC  flc_randomPick ( ::Falcon::VMachine *vm );
