@@ -1,6 +1,6 @@
 /*
    FALCON - The Falcon Programming Language.
-   FILE: time.cpp
+   FILE: seconds.cpp
 
    Time function
    -------------------------------------------------------------------
@@ -15,12 +15,7 @@
 
 #include <falcon/module.h>
 #include <falcon/vm.h>
-
-#ifndef WIN32
-#include <sys/time.h>
-#else
-#include <windows.h>
-#endif
+#include <falcon/sys.h>
 
 namespace Falcon {
 namespace core {
@@ -41,20 +36,9 @@ namespace core {
 
 FALCON_FUNC  seconds ( ::Falcon::VMachine *vm )
 {
-#ifdef WIN32
-   SYSTEMTIME SysTime ;
-   GetSystemTime( &SysTime );
-   vm->retval( SysTime.wHour * 3600.0 + SysTime.wMinute *60 +
-         SysTime.wSecond + SysTime.wMilliseconds /1000.0 );
-
-#else
-   struct timeval time;
-   gettimeofday( &time, 0 );
-   numeric secs = time.tv_sec + (time.tv_usec / 1000000.0 );
-   vm->retval( secs );
-#endif
+   vm->retval( Sys::_seconds() );
 }
 
 }}
 
-/* end of time.cpp */
+/* end of seconds.cpp */
