@@ -890,7 +890,6 @@ void Buffer::destroy( String *str ) const
 
 String::String( uint32 size ):
    m_class( &csh::handler_buffer ),
-   m_id( no_id ),
    m_bExported( false ),
    m_bCore( false )
 {
@@ -902,7 +901,6 @@ String::String( uint32 size ):
 String::String( const char *data ):
    m_class( &csh::handler_static ),
    m_allocated( 0 ),
-   m_id( no_id ),
    m_storage( (byte*) const_cast< char *>(data) ),
    m_bExported( false ),
    m_bCore( false )
@@ -912,7 +910,6 @@ String::String( const char *data ):
 
 String::String( const char *data, int32 len ):
    m_class( &csh::handler_buffer ),
-   m_id( no_id ),
    m_bExported( false ),
    m_bCore( false )
 {
@@ -925,7 +922,6 @@ String::String( const char *data, int32 len ):
 
 String::String( const wchar_t *data ):
    m_allocated( 0 ),
-   m_id( no_id ),
    m_storage( (byte*) const_cast< wchar_t *>(data) ),
    m_bExported( false ),
    m_bCore( false )
@@ -944,7 +940,6 @@ String::String( const wchar_t *data ):
 
 String::String( const wchar_t *data, int32 len ):
    m_allocated( 0 ),
-   m_id( no_id ),
    m_storage( (byte *) const_cast< wchar_t *>( data ) ),
    m_bExported( false ),
    m_bCore( false )
@@ -975,7 +970,6 @@ String::String( const wchar_t *data, int32 len ):
 String::String( const String &other, uint32 begin, uint32 end ):
    m_allocated( 0 ),
    m_size( 0 ),
-   m_id( no_id ),
    m_storage( 0 ),
    m_bExported( false ),
    m_bCore( false )
@@ -1371,7 +1365,7 @@ void String::prepend( uint32 chr )
    setCharAt( 0, chr );
 }
 
-static void uint32ToHex( uint32 number, char *buffer )
+void String::uint32ToHex( uint32 number, char *buffer )
 {
    uint32 divisor = 0x10000000;
    int pos = 0;
@@ -1390,39 +1384,16 @@ static void uint32ToHex( uint32 number, char *buffer )
 
 void String::escape( String &strout ) const
 {
-   int len = length();
-   int pos = 0;
-   strout.m_class->reserve( &strout, len ); // prepare for at least len chars
-   strout.size( 0 ); // clear target string
-
-   while( pos < len )
-   {
-      uint32 chat = getCharAt( pos );
-      switch( chat )
-      {
-         case '"':
-            strout += "\\\""; break;
-         case '\r': strout += "\\r"; break;
-         case '\n': strout += "\\n"; break;
-         case '\t': strout += "\\t"; break;
-         case '\b': strout += "\\b"; break;
-         case '\\': strout += "\\\\"; break;
-         default:
-            if ( chat < 8 ) {
-               char bufarea[12];
-               uint32ToHex( chat, bufarea );
-               strout += bufarea;
-            }
-            else{
-               strout += chat;
-            }
-      }
-      pos++;
-   }
-
+   internal_escape( strout, false );
 }
 
 void String::escapeFull( String &strout ) const
+{
+   internal_escape( strout, true );
+}
+
+
+void String::internal_escape( String &strout, bool full ) const
 {
    int len = length();
    int pos = 0;
@@ -1434,16 +1405,19 @@ void String::escapeFull( String &strout ) const
       uint32 chat = getCharAt( pos );
       switch( chat )
       {
-         case '"':  strout += "\\\""; break;
+         case '"':strout += "\\\""; break;
+         case '\'':strout += "\\\'"; break;
          case '\r': strout += "\\r"; break;
          case '\n': strout += "\\n"; break;
          case '\t': strout += "\\t"; break;
          case '\b': strout += "\\b"; break;
          case '\\': strout += "\\\\"; break;
          default:
-            if ( chat < 8 || chat > 127 ) {
-               char bufarea[12];
-               uint32ToHex( chat, bufarea );
+            if ( chat < 8 || (chat >= 128 && full) ) {
+               char bufarea[14];
+               bufarea[0] = '\\';
+               bufarea[1] = 'x';
+               uint32ToHex( chat, bufarea+2 );
                strout += bufarea;
             }
             else{
@@ -2240,6 +2214,165 @@ bool String::endsWith( const String &str, bool icase ) const
    return true;
 }
 
+bool String::wildcardMatch( const String& wildcard, bool bIcase ) const
+{
+   const String* wcard = &wildcard;
+   const String* cfr = this;
+
+   uint32 wpos = 0, wlen = wcard->length();
+   uint32 cpos = 0, clen = cfr->length();
+
+   uint32 wstarpos = 0xFFFFFFFF;
+
+   while ( cpos < clen )
+   {
+      if( wpos == wlen )
+      {
+         // we have failed the match; but if we had a star, we
+         // may roll back to the starpos and try to match the
+         // rest of the string
+         if ( wstarpos != 0xFFFFFFFF )
+         {
+            wpos = wstarpos;
+         }
+         else {
+            // no way, we're doomed.
+            break;
+         }
+      }
+
+      uint32 wchr = wcard->getCharAt( wpos );
+      uint32 cchr = cfr->getCharAt( cpos );
+
+      switch( wchr )
+      {
+         case '?': // match any character
+            wpos++;
+            cpos++;
+         break;
+
+         case '*':
+         {
+            // mark for restart in case of bad match.
+            wstarpos = wpos;
+
+            // match till the next character
+            wpos++;
+            // eat all * in a row
+            while( wpos < wlen )
+            {
+               wchr = wcard->getCharAt( wpos );
+               if ( wchr != '*' )
+                  break;
+               wpos++;
+            }
+
+            if ( wpos == wlen )
+            {
+               // we have consumed all the chars
+               cpos = clen;
+               break;
+            }
+
+
+            //eat up to next character
+            while( cpos < clen )
+            {
+               cchr = cfr->getCharAt( cpos );
+               if ( cchr == wchr )
+                  break;
+               cpos ++;
+            }
+
+            // we have eaten up the same char? --  then advance also wpos to prepare next loop
+            if ( cchr == wchr )
+            {
+               wpos++;
+               cpos++;
+            }
+            // else, everything must stay as it is, so cpos == clen but wpos != wlen causing fail.
+         }
+         break;
+
+         default:
+            if ( cchr == wchr ||
+                  ( bIcase && cchr < 128 && wchr < 128 && (cchr | 32) == (wchr | 32) )
+               )
+            {
+               cpos++;
+               wpos++;
+            }
+            else
+            {
+               // can we retry?
+               if ( wstarpos != 0xFFFFFFFF )
+                  wpos = wstarpos;
+               else {
+                  // check failed -- we're doomed
+                  return false;
+               }
+            }
+      }
+   }
+
+   // at the end of the loop, the match is ok only if both the cpos and wpos are at the end
+   return wpos == wlen && cpos == clen;
+}
+
+void String::escapeQuotes()
+{
+   uint32 len = length();
+   uint32 i = 0;
+
+   while( i < len )
+   {
+      register uint32 chr =  getCharAt(i);
+      switch( chr )
+      {
+      case '\'': case '\"':
+         insert( i, 0, "\\" );
+         i+=2;
+         ++len;
+         break;
+
+      default:
+         ++i;
+      }
+   }
+}
+
+void String::unescapeQuotes()
+{
+   uint32 len = length();
+   uint32 i = 0;
+   bool state = false;
+
+   while( i < len )
+   {
+      register uint32 chr =  getCharAt(i);
+      switch( chr )
+      {
+      case '\'': case '\"':
+         if( state )
+         {
+            remove( i-1, 1 );
+            --len;
+         }
+         break;
+
+      case '\\':
+         state = true;
+         ++i;
+         break;
+
+      default:
+         state = false;
+         ++i;
+      }
+   }
+}
+
+
 //============================================================
 void string_deletor( void *data )
 {
