@@ -170,10 +170,10 @@ FAL_ERRORDECL( e_switch_decl, 103, "Syntax error in 'switch' statement" );
 FAL_ERRORDECL( e_select_decl, 104, "Syntax error in 'select' statement" );
 FAL_ERRORDECL( e_case_outside, 105, "Statement 'case' is valid only within switch or select statements" );
 FAL_ERRORDECL( e_syn_load, 106, "Syntax error in 'load' directive" );
-FAL_ERRORDECL( e_toplevel_func, 107, "Functions must be declared at toplevel" );
-FAL_ERRORDECL( e_toplevel_obj, 108, "Objects must be declared at toplevel" );
-FAL_ERRORDECL( e_toplevel_class, 109, "Classes must be declared at toplevel" );
-FAL_ERRORDECL( e_toplevel_load, 110, "Load directive must be called at toplevel" );
+FAL_ERRORDECL( e_toplevel_func, 107, "Non-anonymous functions must be declared at top level" );
+FAL_ERRORDECL( e_toplevel_obj, 108, "Objects must be declared at top level" );
+FAL_ERRORDECL( e_toplevel_class, 109, "Classes must be declared at top level" );
+FAL_ERRORDECL( e_toplevel_load, 110, "Load directive must be called at top level" );
 FAL_ERRORDECL( e_syn_while , 111, "Syntax error in 'while' statement" );
 FAL_ERRORDECL( e_syn_if, 112, "Syntax error in 'if' statement" );
 FAL_ERRORDECL( e_syn_else, 113, "Syntax error in 'else' statement" );
