@@ -31,6 +31,7 @@ namespace Falcon
 class Value;
 class Expression;
 class Compiler;
+class Map;
 
 /** Class storing array declarations in source code.
    This class records the content of [ v1, v2 .. vn ] declarations in
@@ -102,13 +103,13 @@ public:
       t_symbol,
       t_symdef,
       t_self,
-      t_sender,
       t_lbind,
 
       t_byref,
       t_array_decl,
       t_dict_decl,
       t_range_decl,
+      t_unbound,
 
       t_expression
    } type_t;
@@ -235,6 +236,7 @@ public:
 
    bool isImmediate() const {
       return m_type == t_nil ||
+             m_type == t_unbound ||
              m_type == t_imm_bool ||
              m_type == t_imm_integer ||
              m_type == t_imm_string ||
@@ -244,7 +246,7 @@ public:
    bool isSimple() const {
       return isImmediate() ||
              m_type == t_symbol || m_type == t_symdef || m_type == t_lbind ||
-             m_type == t_self || m_type == t_sender;
+             m_type == t_self;
    }
 
    bool isTrue() const {
@@ -271,6 +273,7 @@ public:
    Expression *asExpr() const { return m_content.asExpr; }
 
    void setNil() { m_type = t_nil; }
+   void setUnbound() { m_type = t_unbound; }
    void setBool( bool val ) { m_type = t_imm_bool; m_content.asBool = val; }
    void setInteger( int64 val ) { m_type = t_imm_integer; m_content.asInteger = val; }
    void setNumeric( numeric val ) { m_type = t_imm_num; m_content.asNumeric = val; }
@@ -283,10 +286,10 @@ public:
    void setDict( DictDecl *val ) { m_type = t_dict_decl; m_content.asDict = val; }
    void setRange( RangeDecl *val ) { m_type = t_range_decl; m_content.asRange = val; }
    void setSelf() { m_type = t_self; }
-   void setSender() { m_type = t_sender; }
    void setLBind( String *val ) { m_type = t_lbind; m_content.asString = val; }
 
    bool isNil() const { return m_type == t_nil; }
+   bool isUnbound() const { return m_type == t_unbound; }
    bool isBool() const { return m_type == t_imm_bool; }
    bool isInteger() const { return m_type == t_imm_integer; }
    bool isNumeric() const { return m_type == t_imm_num; }
@@ -298,7 +301,6 @@ public:
    bool isArray() const { return m_type == t_array_decl; }
    bool isDict() const { return m_type == t_dict_decl; }
    bool isSelf() const { return m_type == t_self; }
-   bool isSender() const { return m_type == t_sender; }
    bool isRange() const { return m_type == t_range_decl; }
    bool isLBind() const { return m_type == t_lbind; }
 
@@ -385,6 +387,7 @@ public:
       t_lt,
       t_le,
       t_eq,
+      t_exeq,
       t_neq,
 
       t_has,
@@ -456,6 +459,8 @@ public:
    Value *third() const { return m_third; }
    void third( Value *t ) { delete m_third; m_third = t; }
 
+   bool isStandAlone() const;
+
    bool isBinaryOperator() const
    {
       switch( m_operator )
@@ -481,6 +486,7 @@ public:
          case t_le:
          case t_eq:
          case t_neq:
+         case t_exeq:
 
          case t_has:
          case t_hasnt:
@@ -518,6 +524,7 @@ public:
       t_if,
       t_elif,
       t_while,
+      t_loop,
       t_forin,
       t_try,
       t_catch,
@@ -526,9 +533,9 @@ public:
       t_case,
       t_module,
       t_global,
-      t_pass,
 
       t_class,
+      t_state,
       t_function,
       t_propdef,
       t_fordot,
@@ -587,7 +594,7 @@ public:
       Statement( other )
    {}
 
-   Statement *clone() const;
+   StmtNone *clone() const;
 };
 
 
@@ -606,7 +613,7 @@ public:
    SymbolList &getSymbols() { return m_symbols; }
    const SymbolList &getSymbols() const { return m_symbols; }
 
-   virtual Statement *clone() const;
+   virtual StmtGlobal *clone() const;
 };
 
 
@@ -625,7 +632,7 @@ public:
    ~StmtUnref();
 
    Value *symbol() const { return m_symbol; }
-   virtual Statement *clone() const;
+   virtual StmtUnref *clone() const;
 };
 
 class FALCON_DYN_CLASS StmtSelfPrint: public Statement
@@ -642,7 +649,7 @@ public:
 
    ~StmtSelfPrint();
 
-   virtual Statement *clone() const;
+   virtual StmtSelfPrint *clone() const;
 
    ArrayDecl *toPrint() const { return m_toPrint; }
 };
@@ -663,7 +670,7 @@ public:
    virtual ~StmtExpression();
 
    Value *value() const { return m_expr; }
-   virtual Statement *clone() const;
+   virtual StmtExpression *clone() const;
 };
 
 class FALCON_DYN_CLASS StmtFordot: public StmtExpression
@@ -675,7 +682,7 @@ public:
 
    StmtFordot( const StmtFordot &other );
 
-   virtual Statement *clone() const;
+   virtual StmtFordot *clone() const;
 };
 
 class FALCON_DYN_CLASS StmtAutoexpr: public StmtExpression
@@ -687,7 +694,7 @@ public:
 
    StmtAutoexpr( const StmtAutoexpr &other );
 
-   virtual Statement *clone() const;
+   virtual StmtAutoexpr *clone() const;
 };
 
 
@@ -702,7 +709,7 @@ public:
       StmtExpression( other )
    {}
 
-   virtual Statement *clone() const;
+   virtual StmtReturn *clone() const;
 };
 
 
@@ -717,7 +724,7 @@ public:
       StmtExpression( other )
    {}
 
-   virtual Statement *clone() const;
+   virtual StmtLaunch *clone() const;
 };
 
 
@@ -732,30 +739,7 @@ public:
       StmtExpression( other )
    {}
 
-   virtual Statement *clone() const;
-};
-
-
-class FALCON_DYN_CLASS StmtPass: public Statement
-{
-   Value *m_called;
-   Value *m_saveRetIn;
-
-public:
-   StmtPass( uint32 line, Value *exp, Value *in = 0 ):
-      Statement( line, t_pass ),
-      m_called( exp ),
-      m_saveRetIn( in )
-   {}
-
-   StmtPass( const StmtPass &other );
-
-   virtual ~StmtPass();
-
-   Value *called() const { return m_called; }
-   Value *saveIn() const { return m_saveRetIn; }
-
-   virtual Statement *clone() const;
+   virtual StmtRaise *clone() const;
 };
 
 
@@ -778,7 +762,7 @@ public:
    ArrayDecl *objects() const { return m_objects; }
    ArrayDecl *attributes() const { return m_attribs; }
 
-   virtual Statement *clone() const;
+   virtual StmtGive *clone() const;
 };
 
 
@@ -808,7 +792,7 @@ public:
       StmtLoopCtl( other )
    {}
 
-   virtual Statement *clone() const;
+   virtual StmtBreak *clone() const;
 };
 
 class FALCON_DYN_CLASS StmtContinue: public StmtLoopCtl
@@ -826,7 +810,7 @@ public:
    {}
 
    bool dropping() const { return m_dropping; }
-   virtual Statement *clone() const;
+   virtual StmtContinue *clone() const;
 };
 
 
@@ -850,6 +834,7 @@ public:
 
 class FALCON_DYN_CLASS StmtConditional: public StmtBlock
 {
+protected:
    Value *m_condition;
 
 public:
@@ -867,6 +852,22 @@ public:
    // pure virtual
 };
 
+class FALCON_DYN_CLASS StmtLoop: public StmtConditional
+{
+public:
+   StmtLoop( uint32 line, Value *cond = 0):
+      StmtConditional( line, t_loop, cond )
+   {}
+
+   StmtLoop( const StmtLoop &other ):
+      StmtConditional( other )
+   {}
+
+   virtual StmtLoop *clone() const;
+
+   void setCondition( Value *cond ) { m_condition = cond; }
+};
+
 
 class FALCON_DYN_CLASS StmtWhile: public StmtConditional
 {
@@ -879,7 +880,7 @@ public:
       StmtConditional( other )
    {}
 
-   virtual Statement *clone() const;
+   virtual StmtWhile *clone() const;
 };
 
 
@@ -894,7 +895,7 @@ public:
       StmtConditional( other )
    {}
 
-   virtual Statement *clone() const;
+   virtual StmtElif *clone() const;
 };
 
 
@@ -915,7 +916,7 @@ public:
    const StatementList &elifChildren() const { return m_elseifs; }
    StatementList &elifChildren() { return m_elseifs; }
 
-   virtual Statement *clone() const;
+   virtual StmtIf *clone() const;
 };
 
 
@@ -926,10 +927,10 @@ class FALCON_DYN_CLASS StmtForin: public StmtBlock
    StatementList m_middle;
 
    Value *m_source;
-   Value *m_dest;
+   ArrayDecl *m_dest;
 
 public:
-   StmtForin( uint32 line, Value *dest, Value *source ):
+   StmtForin( uint32 line, ArrayDecl *dest, Value *source ):
       StmtBlock( line, t_forin ),
       m_source( source ),
       m_dest( dest )
@@ -947,9 +948,9 @@ public:
    StatementList &middleBlock() { return m_middle; }
 
    Value *source() const { return m_source; }
-   Value *dest() const { return m_dest; }
+   ArrayDecl *dest() const { return m_dest; }
 
-   virtual Statement *clone() const;
+   virtual StmtForin *clone() const;
 };
 
 
@@ -964,7 +965,7 @@ public:
       StmtBlock( other )
    {}
 
-   virtual Statement *clone() const;
+   virtual StmtCaseBlock *clone() const;
 };
 
 
@@ -1028,7 +1029,7 @@ public:
 
    int currentBlock() const { return m_blocks.size(); }
 
-   virtual Statement *clone() const;
+   virtual StmtSwitch *clone() const;
 };
 
 
@@ -1041,7 +1042,7 @@ public:
    StmtSelect( uint32 line, Value *expr );
    StmtSelect( const StmtSelect &other );
 
-   virtual Statement *clone() const;
+   virtual StmtSelect *clone() const;
 };
 
 class FALCON_DYN_CLASS StmtCatchBlock: public StmtBlock
@@ -1060,7 +1061,7 @@ public:
 
    Value *intoValue() const { return m_into; }
 
-   virtual Statement *clone() const;
+   virtual StmtCatchBlock *clone() const;
 };
 
 
@@ -1107,23 +1108,9 @@ public:
 
    int currentBlock() const { return m_handlers.size(); }
 
-   virtual Statement *clone() const;
+   virtual StmtTry *clone() const;
 };
 
-/** Module statement.
-   Actually, this holds an "unnamed block", which may be top-level or anything.
-*/
-/*class FALCON_DYN_CLASS StmtModule: public StmtBlock
-{
-public:
-   StmtModule( uint32 line = 0):
-      StmtBlock( line, t_module )
-   {}
-
-   virtual Statement *clone() const;
-};*/
-
-
 
 class FALCON_DYN_CLASS StmtCallable: public Statement
 {
@@ -1150,6 +1137,7 @@ public:
 };
 
 class StmtFunction;
+class StmtState;
 
 class FALCON_DYN_CLASS StmtClass: public StmtCallable
 {
@@ -1161,6 +1149,10 @@ class FALCON_DYN_CLASS StmtClass: public StmtCallable
    Symbol *m_singleton;
    /** set of expressions (values, usually inherit calls) to be prepended to the constructor */
    ArrayDecl m_initExpressions;
+
+   /** Init state */
+   StmtState *m_initState;
+   StatementList m_states;
 public:
 
    StmtClass( uint32 line, Symbol *name ):
@@ -1168,7 +1160,8 @@ public:
       m_ctor(0),
       m_initGiven( false ),
       m_bDeleteCtor( false ),
-      m_singleton(0)
+      m_singleton(0),
+      m_initState(0)
    {}
 
    StmtClass( const StmtClass &other );
@@ -1195,7 +1188,17 @@ public:
    /** Singleton associated to this class, if any. */
    Symbol *singleton() const { return m_singleton; }
    void singleton( Symbol *s ) { m_singleton = s; }
-   virtual Statement *clone() const;
+   virtual StmtClass *clone() const;
+
+   /** Return the Statement declaring the init state of this class */
+   StmtState* initState() const { return m_initState; }
+
+   /** Sets the init state of this class. */
+   void initState( StmtState* m ) { m_initState = m; }
+
+   /** Return the Statement declaring the init state of this class */
+   bool addState( StmtState* m_state );
+
 };
 
 class FALCON_DYN_CLASS StmtFunction: public StmtCallable
@@ -1232,7 +1235,39 @@ public:
    void setConstructorFor( const StmtClass *cd ) { m_ctor_for = cd; }
    const StmtClass *constructorFor() const { return m_ctor_for; }
 
-   virtual Statement *clone() const;
+   virtual StmtFunction *clone() const;
+};
+
+
+class StmtState: public Statement
+{
+   const String* m_name;
+   StmtClass* m_owner;
+   Map m_funcs;
+   StateDef* m_stateDef;
+
+public:
+
+   StmtState( const String* name, StmtClass* owner );
+   StmtState( const StmtState& other );
+   virtual ~StmtState();
+   virtual StmtState* clone() const;
+
+   /** Functions subscribed to this state, ordered by alias. */
+   const Map& functions() const  { return m_funcs; }
+
+   /** Functions subscribed to this state, ordered by alias. */
+   Map& functions()  { return m_funcs; }
+
+   /** Just a shortcut to insertion in the map.
+    *    Returns false if the map exists.
+    */
+   bool addFunction( const String* name, Symbol* func );
+
+   const String* name() const { return m_name; }
+   StmtClass* owner() const { return m_owner; }
+
+   StateDef* state() const { return m_stateDef; }
 };
 
 class FALCON_DYN_CLASS StmtVarDef: public Statement
@@ -1254,7 +1289,7 @@ public:
    String *name() const { return m_name; }
    Value *value() const { return m_value; }
 
-   virtual Statement *clone() const;
+   virtual StmtVarDef *clone() const;
 };
 
 /** Source File syntactic tree.
