@@ -39,6 +39,7 @@
 #include <falcon/baton.h>
 #include <falcon/livemodule.h>
 #include <falcon/vmcontext.h>
+#include <falcon/mersennetwister.h>
 
 #define FALCON_VM_DFAULT_CHECK_LOOPS 5000
 
@@ -448,6 +449,9 @@ protected:
    /** filtered load path */
    String m_appSearchPath;
 
+   /** random number generator */
+   MTRand _mtrand;
+
    //=============================================================
    // Private functions
    //
@@ -2289,6 +2293,9 @@ public:
    void breakRequest( bool mode ) { m_break = mode; }
    bool breakRequest() const { return m_break; }
 
+   /** Returns the Random Number Generator */
+   MTRand& getRNG(void) { return _mtrand; }
+
 //==========================================================================
 //==========================================================================
 //==========================================================================
