@@ -58,7 +58,7 @@ static const String &internal_getErrorMsg( VMachine *vm, int error_code )
 
 FALCON_FUNC ZLib_getVersion( ::Falcon::VMachine *vm )
 {
-   GarbageString *gsVersion = new GarbageString( vm, zlibVersion() );
+   CoreString *gsVersion = new CoreString( zlibVersion() );
    gsVersion->bufferize();
    vm->retval( gsVersion );
 }
@@ -86,9 +86,8 @@ FALCON_FUNC ZLib_compress( ::Falcon::VMachine *vm )
    Item *dataI = vm->param( 0 );
    if ( dataI == 0 || ( ! dataI->isString() && ! dataI->isMemBuf() ) )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ )
-                                        .extra( "S|M" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ )
+                                        .extra( "S|M" ) );
    }
 
    int err;
@@ -131,10 +130,9 @@ FALCON_FUNC ZLib_compress( ::Falcon::VMachine *vm )
    if ( err != Z_OK )
    {
       // as err is < 0, we reverse it.
-      vm->raiseModError( new ZLibError(
+      throw new ZLibError(
             ErrorParam( FALCON_ZLIB_ERROR_BASE - err, __LINE__ )
-            .desc( internal_getErrorMsg( vm, err ) ) ) );
-      return;
+            .desc( internal_getErrorMsg( vm, err ) ) );
    }
 
 
@@ -145,7 +143,7 @@ FALCON_FUNC ZLib_compress( ::Falcon::VMachine *vm )
       allocLen = compLen;
    }
 
-   MemBuf *result = new MemBuf_1( vm, compData, allocLen, true );
+   MemBuf *result = new MemBuf_1( compData, allocLen, memFree );
    vm->retval( result );
 }
 
@@ -165,9 +163,8 @@ FALCON_FUNC ZLib_compressText( ::Falcon::VMachine *vm )
    Item *dataI = vm->param( 0 );
    if ( dataI == 0 || ! dataI->isString() )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ )
-                  .extra( "S" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ )
+                  .extra( "S" ) );
    }
 
    int err;
@@ -182,11 +179,11 @@ FALCON_FUNC ZLib_compressText( ::Falcon::VMachine *vm )
 
    allocLen = compLen; // and keep a copy.
    compData = (Bytef *) memAlloc( compLen );
-   compData[0] = dataI->asString()->manipulator()->charSize();
-   compData[1] = dataLen >> 24;
-   compData[2] = (dataLen >> 16) & 0xff;
-   compData[3] = (dataLen >> 8) & 0xff;
-   compData[4] = (dataLen) & 0xff;
+   compData[0] = (Bytef) dataI->asString()->manipulator()->charSize();
+   compData[1] = (Bytef)(dataLen >> 24);
+   compData[2] = (Bytef)((dataLen >> 16) & 0xff);
+   compData[3] = (Bytef)((dataLen >> 8) & 0xff);
+   compData[4] = (Bytef)((dataLen) & 0xff);
    compLen-=5;
 
    do {
@@ -198,11 +195,11 @@ FALCON_FUNC ZLib_compressText( ::Falcon::VMachine *vm )
          memFree( compData );
          compLen += dataLen/2;
          allocLen = compLen;
-         compData[0] = dataI->asString()->manipulator()->charSize();
-         compData[1] = dataLen >> 24;
-         compData[2] = (dataLen >> 16) & 0xff;
-         compData[3] = (dataLen >> 8) & 0xff;
-         compData[4] = (dataLen) & 0xff;
+         compData[0] = (Bytef) dataI->asString()->manipulator()->charSize();
+         compData[1] = (Bytef)(dataLen >> 24);
+         compData[2] = (Bytef)((dataLen >> 16) & 0xff);
+         compData[3] = (Bytef)((dataLen >> 8) & 0xff);
+         compData[4] = (Bytef)((dataLen) & 0xff);
          compLen-=5;
 
          compData = (Bytef *) memAlloc( compLen );
@@ -214,9 +211,8 @@ FALCON_FUNC ZLib_compressText( ::Falcon::VMachine *vm )
 
    if ( err != Z_OK )
    {
-      vm->raiseModError( new ZLibError( ErrorParam( FALCON_ZLIB_ERROR_BASE -err, __LINE__ )
-               .desc( internal_getErrorMsg( vm, err ) ) ) );
-      return;
+      throw new ZLibError( ErrorParam( FALCON_ZLIB_ERROR_BASE -err, __LINE__ )
+               .desc( internal_getErrorMsg( vm, err ) ) );
    }
 
 
@@ -227,7 +223,7 @@ FALCON_FUNC ZLib_compressText( ::Falcon::VMachine *vm )
       allocLen = compLen + 5;
    }
 
-   MemBuf *result = new MemBuf_1( vm, compData, allocLen, true );
+   MemBuf *result = new MemBuf_1( compData, allocLen, memFree );
    vm->retval( result );
 }
 
@@ -245,9 +241,8 @@ FALCON_FUNC ZLib_uncompress( ::Falcon::VMachine *vm )
 
    if ( dataI == 0 || ( ! dataI->isString() && ! dataI->isMemBuf() ) )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ )
-               .extra( "S|M" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ )
+               .extra( "S|M" ) );
    }
 
    int err;
@@ -269,7 +264,7 @@ FALCON_FUNC ZLib_uncompress( ::Falcon::VMachine *vm )
    }
 
    // preallocate a good default memory
-   compLen = sizeof(char) * ( dataInSize * 4 );
+   compLen = sizeof(char) * ( dataInSize * 2 );
    if ( compLen < 512 )
    {
       compLen = 512;
@@ -282,12 +277,12 @@ FALCON_FUNC ZLib_uncompress( ::Falcon::VMachine *vm )
    {
       err = uncompress( compData, &compLen, dataIn, dataInSize );
 
-      if ( err == Z_MEM_ERROR )
+      if ( err == Z_BUF_ERROR )
       {
-         //TODO: break also with Z_MEM_ERROR if we're using too much memory, like i.e. 512MB
+         //TODO: break also with Z_STREAM_ERROR if we're using too much memory, like i.e. 512MB
 
          // try with a larger buffer
-         compLen += dataInSize < 512 ? 512 : dataInSize * 4;
+         compLen += dataInSize < 512 ? 512 : dataInSize * 2;
          allocLen = compLen;
          memFree( compData );
          compData = (Bytef *) memAlloc( compLen );
@@ -298,10 +293,9 @@ FALCON_FUNC ZLib_uncompress( ::Falcon::VMachine *vm )
 
    if ( err != Z_OK )
    {
-      vm->raiseModError( new ZLibError(
+      throw new ZLibError(
                ErrorParam( FALCON_ZLIB_ERROR_BASE-err, __LINE__ )
-               .desc( internal_getErrorMsg( vm, err ) ) ) );
-      return;
+               .desc( internal_getErrorMsg( vm, err ) ) );
    }
 
    if ( compLen < allocLen )
@@ -310,7 +304,7 @@ FALCON_FUNC ZLib_uncompress( ::Falcon::VMachine *vm )
       allocLen = compLen;
    }
 
-   MemBuf *result = new MemBuf_1( vm, compData, allocLen, true );
+   MemBuf *result = new MemBuf_1( compData, allocLen, memFree );
    vm->retval( result );
 }
 
@@ -330,13 +324,12 @@ FALCON_FUNC ZLib_uncompressText( ::Falcon::VMachine *vm )
 
    if ( dataI == 0 || ( ! dataI->isString() && ! dataI->isMemBuf() ) )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ )
-                                        .extra( "S|M" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ )
+                                        .extra( "S|M" ) );
    }
 
    int err;
-   uLong allocLen, compLen;
+   uLong compLen;
    Bytef *compData;
    const byte *dataIn;
    uint32 dataInSize;
@@ -356,9 +349,8 @@ FALCON_FUNC ZLib_uncompressText( ::Falcon::VMachine *vm )
    // type of string
    if ( dataIn[0] != 1 && dataIn[0] != 2 && dataIn[0] != 4 )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ )
-               .extra( FAL_STR(zl_msg_notct) ) ) );
-      return;
+      throw new ZLibError( ErrorParam( FALCON_ZLIB_ERROR_BASE, __LINE__ ).
+               desc( FAL_STR(zl_msg_notct) ) );
    }
 
    // get length
@@ -369,13 +361,12 @@ FALCON_FUNC ZLib_uncompressText( ::Falcon::VMachine *vm )
 
    if ( err != Z_OK )
    {
-      vm->raiseModError( new ZLibError(
+      throw new ZLibError(
                ErrorParam( FALCON_ZLIB_ERROR_BASE-err, __LINE__ )
-               .desc( internal_getErrorMsg( vm, err ) ) ) );
-      return;
+               .desc( internal_getErrorMsg( vm, err ) ) );
    }
 
-   GarbageString *result = new GarbageString( vm );
+   CoreString *result = new CoreString;
    result->adopt( (char *) compData, compLen, compLen );
    // set correct manipulator
    if (dataIn[0] == 2 )
@@ -405,16 +396,11 @@ FALCON_FUNC ZLib_uncompressText( ::Falcon::VMachine *vm )
    See the Error class in the core module.
 */
 
-/*#
-   @init ZLibError
-   @brief Initializes the zlib error.
-*/
-
 FALCON_FUNC  ZLibError_init ( ::Falcon::VMachine *vm )
 {
    CoreObject *einst = vm->self().asObject();
    if( einst->getUserData() == 0 )
-      einst->setUserData( new Falcon::ErrorCarrier( new ZLibError ) );
+      einst->setUserData( new ZLibError );
 
    ::Falcon::core::Error_init( vm );
 }
