@@ -56,7 +56,7 @@ public:
    virtual bool errorDescription( ::Falcon::String &description ) const;
    virtual int64 lastError(void) const;
 
-   virtual UserData *clone() const;
+   virtual FalconData *clone() const;
 };
 }
 
